@@ -34,6 +34,7 @@ from pwem.emlib.image import ImageHandler
 from tomo.protocols import ProtTomoBase
 from tomo.convert import writeTiStack
 from tomo.objects import Tomogram, TiltSeries
+from novactf import Plugin
 
 
 class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
@@ -56,7 +57,24 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                       important=True,
                       label='Input set of tilt-Series')
 
-        form.addParam('tomoThickness', params.FloatParam,
+        form.addParam('ctfEstimationType',
+                      params.EnumParam,
+                      choices=['IMOD', 'Other'],
+                      default=1,
+                      label='CTF estimation',
+                      important=True,
+                      display=params.EnumParam.DISPLAY_HLIST,
+                      help='CTF estimation origin.')
+
+        form.addParam('protImodCtfEstimation',
+                      params.PointerParam,
+                      label="IMOD CTF estimation run",
+                      condition='ctfEstimationType==0',
+                      pointerClass='ProtCtfEstimation',
+                      help='Select the previous IMOD CTF estimation run.')
+
+        form.addParam('tomoThickness',
+                      params.FloatParam,
                       default=100,
                       label='Tomogram thickness',
                       important=True,
@@ -112,7 +130,10 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
     def _insertAllSteps(self):
         for ts in self.inputSetOfTiltSeries.get():
             self._insertFunctionStep('convertInputStep', ts.getObjId())
-            self._insertFunctionStep('generateCtffindDefocusFile', ts.getObjId())
+            if self.ctfEstimationType.get() == 0:
+                self._insertFunctionStep('generateImodDefocusFile', ts.getObjId())
+            elif self.ctfEstimationType.get() == 1:
+                self._insertFunctionStep('generateCtffindDefocusFile', ts.getObjId())
             self._insertFunctionStep('computeDefocusStep', ts.getObjId())
             self._insertFunctionStep('computeCtfCorrectionStep', ts.getObjId())
             self._insertFunctionStep('computeFlipStep', ts.getObjId())
@@ -137,6 +158,14 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
         angleFilePath = os.path.join(tmpPrefix, "%s.rawtlt" % tsId)
         ts.generateTltFile(angleFilePath)
 
+    def generateImodDefocusFile(self, tsObjId):
+        ts = self.inputSetOfTiltSeries.get()[tsObjId]
+        tsId = ts.getTsId()
+        outputDefocusFile = self.getDefocusFile(ts)
+
+        outputDefocusFilePrefix = self.protImodCtfEstimation.get()._getExtraPath(tsId)
+        path.copyFile(os.path.join(outputDefocusFilePrefix, "%s.defocus" % tsId), outputDefocusFile)
+
     def generateCtffindDefocusFile(self, tsObjId):
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
         outputDefocusFile = self.getDefocusFile(ts)
@@ -162,7 +191,6 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                        (counter+1, vector[0], vector[1], vector[2], vector[3], vector[4], vector[5])
                 f.writelines(line)
 
-
     def computeDefocusStep(self, tsObjId):
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
         tsId = ts.getTsId()
@@ -196,7 +224,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                       "-PixelSize %(PixelSize)s " \
                       "-DefocusStep %(DefocusStep)d"
 
-        self.runJob('/home/fede/novaCTF/novaCTF', argsDefocus % paramsDefocus)
+        Plugin.runNovactf(self, 'novaCTF', argsDefocus % paramsDefocus)
 
     def computeCtfCorrectionStep(self, tsObjId):
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
@@ -236,7 +264,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                                 "-Cs %(SphericalAberration)f " \
                                 "-Volt %(Voltage)d"
 
-            self.runJob('/home/fede/novaCTF/novaCTF', argsCtfCorrection % paramsCtfCorrection)
+            Plugin.runNovactf(self, 'novaCTF', argsCtfCorrection % paramsCtfCorrection)
 
             i += 1
 
@@ -249,7 +277,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
         i = 0
         while os.path.exists(os.path.join(inputFilePath + str(i))):
             argsFlip = inputFilePath + str(i) + " " + outputFilePath + str(i)
-            self.runJob('clip flipyz', argsFlip)
+            Plugin.runImod(self, 'clip flipyz', argsFlip)
             i += 1
 
     def computeFilteringStep(self, tsObjId):
@@ -276,7 +304,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                                     "-TILTFILE %(TiltFile)s " \
                                     "-StackOrientation %(StackOrientation)s " \
                                     "-RADIAL %(Radial)s"
-            self.runJob('/home/fede/novaCTF/novaCTF', argsFilterProjections % paramsFilterProjections)
+            Plugin.runNovactf(self, 'novaCTF', argsFilterProjections % paramsFilterProjections)
 
             i += 1
 
@@ -310,7 +338,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
                     "-PixelSize %(PixelSize)f " \
                     "-DefocusStep %(DefocusStep)d"
 
-        self.runJob('/home/fede/novaCTF/novaCTF', args3dctf % params3dctf)
+        Plugin.runNovactf(self, 'novaCTF', args3dctf % params3dctf)
 
     def createOutputStep(self, tsObjId):
         ts = self.inputSetOfTiltSeries.get()[tsObjId]
@@ -358,7 +386,7 @@ class ProtTomoCtfReconstruction(EMProtocol, ProtTomoBase):
     def _validate(self):
         validateMsgs = []
 
-        if not self.inputSetOfTiltSeries.get().getFirstItem().getFirstItem().hasCTF():
+        if self.ctfEstimationType.get() == 1 and not self.inputSetOfTiltSeries.get().getFirstItem().getFirstItem().hasCTF():
             validateMsgs = "You need to generate an estimation of the CTF associated to the tilt series to calculate " \
                            "its corrected reconstruction"
 
